package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogModel;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecStateItem;
import javafx.collections.ListChangeListener;
import javafx.scene.control.Button;
import javafx.scene.control.Control;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

class ESpecStateButton extends VBox {
    private final ESpecStateItem eSpecStateItem;

    private final Button mainButton;
    private final HBox extraButtonsBox;

    public ESpecStateButton(@Nonnull ESpecStateItem eSpecStateItem) {
        this.eSpecStateItem = eSpecStateItem;

        this.extraButtonsBox = new HBox();
        this.mainButton = createBaseButton(eSpecStateItem);

        getChildren().addAll(mainButton, extraButtonsBox);

        eSpecStateItem.getChildren().addListener(
                (ListChangeListener<ESpecStateItem>) (c) -> {
                    while (c.next()) {
                        if (c.wasAdded()) {
                            for (ESpecStateItem i : c.getAddedSubList())
                                addSubButton(i);
                        }
                    }
                });

//        eSpecStateItem.nameProperty().addListener((o, oldVal, newVal) -> updateName());
//        eSpecStateItem.getChildren().addListener(
//                (ListChangeListener<ESpecStateItem>) (change) -> updateName());
//        updateName();
    }

    public Button createBaseButton(ESpecStateItem eSpecStateItem) {
        Button res = new Button();
        res.getStylesheets().add(getClass().getResource("espec-state-item.css").toExternalForm());
        res.getStyleClass().addAll("especState");
        res.textProperty().bind(eSpecStateItem.nameProperty());
        eSpecStateItem.statusProperty().addListener(
                (o, oldVal, newVal) -> updateStatus(newVal, res)
        );
        updateStatus(eSpecStateItem.getStatus(), res);
        return res;
    }

    public void addSubButton(ESpecStateItem eSpecStateItem) {
        extraButtonsBox.getChildren().add(createBaseButton(eSpecStateItem));
    }

//    public void updateName() {
//        String newText = eSpecStateItem.getName();
//        if (!eSpecStateItem.getChildren().isEmpty()) {
//            newText += eSpecStateItem.getChildren().stream()
//                    .map(ESpecStateItem::getName)
//                    .collect(Collectors.joining(", "));
//            getChildren().add(new Label("Test"));
//        }
//        this.mainButton.setText(newText);
//    }

    public void updateStatus(ESpecStateItem.ESpecStateStatus status, Control target) {
        List<String> oldStyleClasses = new ArrayList(Arrays.asList("future", "running", "fail", "success"));
        List<String> newStyleClasses = new ArrayList();

        switch (status) {
            case FAILURE: {
                oldStyleClasses.remove("fail");
                newStyleClasses.add("fail");
                break;
            }
            case FUTURE: {
                oldStyleClasses.remove("future");
                newStyleClasses.add("future");
                break;
            }
            case SUCCESS: {
                oldStyleClasses.remove("success");
                newStyleClasses.add("success");
                break;
            }
            case RUNNING: {
                oldStyleClasses.remove("running");
                newStyleClasses.add("running");
                break;
            }
        }

        target.getStyleClass().removeAll(oldStyleClasses);
        newStyleClasses.removeAll(target.getStyleClass());
        target.getStyleClass().addAll(newStyleClasses);
    }
}
